package cn.com.ctop.finance.payment.controller;

import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.finance.payment.entity.FinancePaymentRechargeApplication;
import cn.com.ctop.finance.payment.service.IFinancePaymentRechargeApplicationService;
import com.alibaba.fastjson.JSONObject;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.modules.system.service.ISysRoleService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

/**
 * 充值申请表
 */
@Slf4j
@Api(tags = "充值申请表")
@RestController
@RequestMapping("/financePayment/rechargeApplication")
public class FinancePaymentRechargeApplicationController {
    @Autowired
    private IFinancePaymentRechargeApplicationService financePaymentRechargeApplicationService;
    @Autowired
    private ISysRoleService sysRoleService;

    /**
     * 分页列表查询
     */
    @ApiOperation(value = "充值申请表-分页列表查询", notes = "充值申请表-分页列表查询")
    @GetMapping(value = "/list")
    public Result<Object> queryPageList(FinancePaymentRechargeApplication recharge,
                                        @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                        @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize) {
        try {
            //查询用户角色
            String roleCode = sysRoleService.getRoleCodeByUserId(recharge.getUserId());
            if ("sale".equals(roleCode) || "saleAssistant".equals(roleCode) || "saleAm".equals(roleCode)) {
                recharge.setApplicanterId(recharge.getUserId());
            }
            return financePaymentRechargeApplicationService.queryPageList(recharge, pageNo, pageSize);
        } catch (Exception e) {
            e.printStackTrace();
            return Result.error("查询失败！");
        }
    }

    /**
     * 查询充值账户详情
     */
    @GetMapping(value = "/queryRechargeAccountDetails")
    public Result<Object> queryRechargeAccountDetails(Long rechargeId) {
        try {
            return financePaymentRechargeApplicationService.queryRechargeAccountDetails(rechargeId);
        } catch (Exception e) {
            e.printStackTrace();
            return Result.error("查询失败！");
        }
    }

    /**
     * 添加
     */
    @PostMapping(value = "/add")
    public Result add(@RequestBody JSONObject data) {
        try {
            return financePaymentRechargeApplicationService.add(data);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
        return Result.error("新增异常");
    }

    /**
     * 编辑
     */
    @PostMapping(value = "/edit")
    public Result<Object> edit(@RequestBody JSONObject data) {
        try {
            return financePaymentRechargeApplicationService.edit(data);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
        return Result.error("修改异常");
    }

    /**
     * 更改状态
     */
    @PostMapping(value = "/updateStatus")
    public Result<Object> updateStatus(@RequestBody JSONObject data) {
        try {
            return financePaymentRechargeApplicationService.updateStatus(data);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
        return Result.error("修改异常");
    }

    /**
     * 通过id删除
     */
    @GetMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id") String id) {
        try {
            financePaymentRechargeApplicationService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 通过id查询
     */
    @GetMapping(value = "/queryById")
    public Result<FinancePaymentRechargeApplication> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<FinancePaymentRechargeApplication> result = new Result<>();
        FinancePaymentRechargeApplication financePaymentRechargeApplication = financePaymentRechargeApplicationService.getById(id);
        if (financePaymentRechargeApplication == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(financePaymentRechargeApplication);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 上传临时文件，更新状态
     */
    @PostMapping(value = "/updateTemporaryFiles")
    public Result<Object> updateTemporaryFiles(@RequestBody JSONObject data) {
        try {
            return financePaymentRechargeApplicationService.updateTemporaryFiles(data);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return Result.error("数据异常");
    }

    /**
     * 查询审核流程
     */
    @GetMapping(value = "/queryAuditProcess")
    public Result<Object> queryAuditProcess(Long id) {
        try {
            if (Check.isNull(id)) {
                return Result.error("缺少参数");
            }
            return financePaymentRechargeApplicationService.queryAuditProcess(id);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return Result.error("查询异常");
    }

}
